//#########################################################################
// File: LCD.H
//
//#########################################################################
// Last change: 02.04.2010
//#########################################################################
// Compiler: AVR-GCC 4.1.1
//#########################################################################
// hk@holger-klabunde.de
//#########################################################################
//@{

#ifndef __LCD_H
#define __LCD_H

#include <avr/pgmspace.h>

// You should use only ONE of these !
#define HD44780
//#define KS073
//#define KS066		//not tested
//#define KS070		//not tested

#define LCD_LINES	4
#define LCD_ROWS	20
// Note: If you have a 1x16 display, you may have to use LCD_LINES 2 and LCD_ROWS 8.
// Most 1x16 displays are 2x8 displays organized in one line.

// If you want to use your LCD for example at PORTF you can make access atomic here
//#define INTERFACE_ATOMIC_DATA	// use atomic access to the lcd data port
//#define INTERFACE_ATOMIC_E	// use atomic access to E signal
//#define INTERFACE_ATOMIC_RS	// use atomic access to RS signal

//#define LCD_BUSY_CHECK 	  // not implemented, RW has to be connected to ground

#define USE_LCD_RW	1	// nicht wirklich;)

//#define EADIP204     //EADIP204 Zeichensatzkonvertierungen
//#define CUSTOM_TABLE //Eigene Tabelle. In ConvertChar() selbst definieren
//Wenn beide nicht definiert werden wird eine Standardkonvertierung benutzt

// Some usefull defines
#define NOP	 asm volatile ("nop" ::)
#define sbi(portn, bitn) ((portn)|=(1<<(bitn)))
#define cbi(portn, bitn) ((portn)&=~(1<<(bitn)))

#if USE_LCD_RW
 #define LCD_RW_BIT		5
 #define LCD_RW_PORT	PORTD
 #define LCD_RW_DDR		DDRD

 #define LCD_RW_ON 		sbi(LCD_RW_PORT, LCD_RW_BIT)
 #define LCD_RW_OFF		cbi(LCD_RW_PORT, LCD_RW_BIT)
#endif

#define LCD_RS_BIT		4 //LCD Register Select Pin
#define LCD_RS_PORT		PORTD
#define LCD_RS_DDR		DDRD

#define LCD_E_BIT		6 //LCD Chip Select Pin
#define LCD_E_PORT		PORTD
#define LCD_E_DDR		DDRD

#define LCD_D4_BIT		0
#define LCD_D4_DDR		DDRD
#define LCD_D4_PORT		PORTD

#define LCD_D5_BIT		1
#define LCD_D5_DDR		DDRD
#define LCD_D5_PORT		PORTD

#define LCD_D6_BIT		2
#define LCD_D6_DDR		DDRD
#define LCD_D6_PORT		PORTD

#define LCD_D7_BIT		3
#define LCD_D7_DDR		DDRD
#define LCD_D7_PORT		PORTD

#define LCD_E_ON		sbi(LCD_E_PORT, LCD_E_BIT)
#define LCD_E_OFF		cbi(LCD_E_PORT, LCD_E_BIT)
#define LCD_RS_ON		sbi(LCD_RS_PORT, LCD_RS_BIT)
#define LCD_RS_OFF		cbi(LCD_RS_PORT, LCD_RS_BIT)

void LCDInit(void);
void LCDCls(void);
void LCDPos(unsigned char, unsigned char);
void LCDSetCGRAM(unsigned char adr, unsigned char *buf);
void LCDWriteByte(unsigned char);
void LCDWriteNibble(unsigned char);
void LCDWrite(char *);

void LCDWrite_p(const char *s);
#define LCDWrite_P(text)   LCDWrite_p(PSTR(text))

char ConvertChar(char data);

void Delay1ms(unsigned int time);
void Delay1us(unsigned int time);

void LCDHexChar(unsigned char by);
void LCDHexInt(unsigned int by);
void LCDHexLong(unsigned long l);

// Some security checks !
#if defined (HD44780) && defined (KS073)
 #error "Define HD44780 or KS073 only in lcd.h, NOT both !"
#endif

#if defined (HD44780) && defined (KS066)
 #error "Define HD44780 or KS066 only in lcd.h, NOT both !"
#endif

#if defined (KS066) && defined (KS073)
 #error "Define KS066 or KS073 only in lcd.h, NOT both !"
#endif

#if !defined (HD44780) && !defined (KS066) && !defined (KS070) && !defined (KS073)
 #error "Define at least HD44780 or KS066 or KS070 or KS073 in lcd.h !"
#endif

#endif
//@}
